package edu.gmu.calc.test;

import edu.gmu.calc.RpnCalculator;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;

public class RpnCalculatorTest {

    private RpnCalculator calc;

    @Before
    public void setup() {
        calc = new RpnCalculator();
        calc.push(2);
        calc.push(5);
        calc.push(9);
    }

    @Test
    public void add() {
        calc.apply(RpnCalculator.Operator.ADD);
        Assert.assertEquals(14, calc.peek(), 0);
    }

    @Test
    public void subtract() {
        calc.apply(RpnCalculator.Operator.SUBTRACT);
        Assert.assertEquals(-4, calc.peek(), 0);
    }

    @Test
    public void multiply() {
        calc.apply(RpnCalculator.Operator.MULTIPLY);
        Assert.assertEquals(45, calc.peek(), 0);
    }

    @Test
    public void divide() {
        calc.apply(RpnCalculator.Operator.DIVIDE);
        Assert.assertEquals(5.0 / 9.0, calc.peek(), 0);
    }

    @Test
    public void isOperator() {
        Assert.assertTrue(RpnCalculator.isOperator("+"));
        Assert.assertTrue(RpnCalculator.isOperator("-"));
        Assert.assertTrue(RpnCalculator.isOperator("*"));
        Assert.assertTrue(RpnCalculator.isOperator("/"));
        Assert.assertFalse(RpnCalculator.isOperator("30"));
    }

    @Test
    public void parseOperator() {
        Assert.assertEquals(RpnCalculator.Operator.ADD, RpnCalculator.parseOperator("+"));
        Assert.assertEquals(RpnCalculator.Operator.SUBTRACT, RpnCalculator.parseOperator("-"));
        Assert.assertEquals(RpnCalculator.Operator.MULTIPLY, RpnCalculator.parseOperator("*"));
        Assert.assertEquals(RpnCalculator.Operator.DIVIDE, RpnCalculator.parseOperator("/"));
    }

    @Test
    public void isOperand() {
        Assert.assertTrue(RpnCalculator.isOperand("3.0"));
        Assert.assertTrue(RpnCalculator.isOperand("9"));
        Assert.assertFalse(RpnCalculator.isOperand("+"));
        Assert.assertFalse(RpnCalculator.isOperand("-"));
    }

    @Test
    public void parseOperand() {
        Assert.assertEquals(3.0, RpnCalculator.parseOperand("3.0"), 0);
    }

    @Test
    public void isQuit() {
        Assert.assertTrue(RpnCalculator.isQuit("q"));
        Assert.assertTrue(RpnCalculator.isQuit("Q"));
    }

    @Test
    public void mixedOps() throws IOException {
        String input = "5\n2\n*\n40\n+\n20\n-\n10\n/\n";
        RpnCalculator calc = new RpnCalculator();

        RpnCalculator.parse(calc, new BufferedReader(new StringReader(input)));

        Assert.assertEquals(3.0, calc.peek(), 0);

    }
}
